/*

    qmailalizer - a qmail-send log analysis program

    Copyright (C) 2001  Pichler Wolfgang  (wpichler@dialog-gruppe.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.


*/

#ifndef RESULTS_H
#include "results.h"
#ifndef MESSAGE_H
#include "message.h"
#endif
#include <math.h>
#ifndef GRAPHS_H
#include "graphs.h"
#endif
#include <string.h>
#include "stuff.h"

Results::Results (MHandler *messages_p,Images *images_p,Config *config_p)
{
  messages = messages_p;
  images = images_p;
  config = config_p;
  dWurzel = NULL;
  ldWurzel = NULL;
  firstmsg = NULL;
  lastmsg = NULL;
  load_virtual_domains ();
}

Results::~Results ()
{ LDomains *lldomain,*aktldomain;
  Domains *ldomain,*aktdomain;
  Members *lmember,*aktmember;
  MContainer *lmcontainer,*aktmcontainer;
  
  // Zuerst die LDomains freigeben
  aktldomain = ldWurzel;
  while (aktldomain != NULL) {
    lldomain = aktldomain->next;
    if (aktldomain->unixuser != NULL)
      free(aktldomain->unixuser);
    if (aktldomain->domain != NULL)
      free(aktldomain->domain);
    free (aktldomain);
    aktldomain = lldomain;
  }
  ldWurzel = NULL;
  
  // Jetzt kommen die domains inc members
  aktdomain = dWurzel;
  while (aktdomain != NULL) {
    ldomain = aktdomain->next;
    aktmember = aktdomain->members;
    // Die member der Domain freigeben
    while (aktmember != NULL) {
      lmember = aktmember->next;
      if (aktmember->name != NULL)
        free (aktmember->name);
      if (aktmember->start != NULL)
        free (aktmember->start);
      if (aktmember->end != NULL)
        free (aktmember->end);
      free (aktmember);
      aktmember = lmember;
    }
    // und auch die message container
    aktmcontainer = aktdomain->messages;
    while (aktmcontainer != NULL) {
      lmcontainer = aktmcontainer->next;
      free (aktmcontainer);
      aktmcontainer = lmcontainer;
    }
    if (aktdomain->domainname != NULL)
      free (aktdomain->domainname);
    if (aktdomain->filename != NULL)
      free (aktdomain->filename);
    if (aktdomain->link != NULL)
      free (aktdomain->link);
    if (aktdomain->start != NULL)
      free (aktdomain->start);
    if (aktdomain->end != NULL)
      free (aktdomain->end);
    free (aktdomain);
    aktdomain = ldomain;
  }
  dWurzel = NULL;
}

int Results::anzmessages ()
{ Message *amessage;
  int i;
  
  i = 0;
  amessage = messages->mWurzel;
  while (amessage != NULL) {
    i++;
    amessage = amessage->next;
  }
  return i;
}

int Results::anzdeliveries ()
{ Message *amessage;
  Deliveries *adeliverie;
  int i=0;
  
  amessage = messages->mWurzel;
  while (amessage != NULL) {
    if ((adeliverie = amessage->deliveries) != NULL)
      while (adeliverie != NULL) {
        i++;
	adeliverie = adeliverie->next;
      }
    amessage = amessage->next;   
  }
  return i;
}

struct tm * Results::firstmessage ()
{ struct tm *ftime = NULL;
  Message *amessage = messages->mWurzel;
  
  if (firstmsg == NULL) {
    while (amessage != NULL) {
      if (ftime == NULL)
        ftime = amessage->startzeit;
      else
        if (mktime(ftime) > mktime(amessage->startzeit))
          ftime = amessage->startzeit;
      amessage = amessage->next;
    }
    firstmsg = ftime;
  }
  else
    ftime = firstmsg;
  return ftime;
}

struct tm * Results::lastmessage ()
{ struct tm *ftime = NULL;
  Message *amessage = messages->mWurzel;
  
  if (lastmsg == NULL) {
    while (amessage != NULL) {
      if (ftime == NULL)
        ftime = amessage->startzeit;
      else
        if (mktime(ftime) < mktime(amessage->startzeit))
          ftime = amessage->startzeit;
      amessage = amessage->next;
    }
    lastmsg = ftime;
  }
  else
    ftime = lastmsg;
  return ftime;
}

int Results::diffdays()
{
  return abs((mktime(lastmessage())-mktime(firstmessage())) / 86400);
}

int Results::diffhours()
{
  return abs((mktime(lastmessage())-mktime(firstmessage())) / 3600) - diffdays()*24;
}

int Results::diffminutes()
{
  return abs((mktime(lastmessage())-mktime(firstmessage())) / 60) - diffdays()*24*60 - diffhours()*60;
}

int Results::diffseconds()
{
  return absdiffsec() - diffdays()*24*60*60 - diffhours()*60*60 - diffminutes()*60;
}

int Results::absdiffsec ()
{
  return mktime(lastmsg)-mktime(firstmsg);
}

float Results::emailspersecond()
{ float a,b;

  a = anzmessages();
  b = absdiffsec();
  return a/b;
}

int Results::emailintervall ()
{ float temp;

  temp = 1 / emailspersecond();
  return int(temp);
}

int Results::perhour(time_t startzeit)
{ time_t start,end;
  Message *amessage=messages->mWurzel;
  int i = 0;
  
  start = startzeit;
  // plus einer Stunde
  end = start + 60*60;
  // Jetzt die EMails zhlen die in diesem Zeitraum liegen
  while (amessage != NULL) {
    if ((mktime(amessage->startzeit) >= start) && (mktime(amessage->startzeit) < end))
      i++;
    amessage = amessage->next;
  }
  return i;
}

int Results::d_perhour(time_t startzeit)
{ time_t start,end;
  Message *amessage=messages->mWurzel;
  Deliveries *adeliverie;
  int i = 0;
  
  start = startzeit;
  // plus einer Stunde
  end = start + 60*60;
  // Jetzt die EMails zhlen die in diesem Zeitraum liegen
  while (amessage != NULL) {
    if ((adeliverie = amessage->deliveries) != NULL)
    {
      while (adeliverie != NULL) {
        if ((mktime(amessage->startzeit) >= start) && (mktime(amessage->startzeit) < end))
          i++;
	adeliverie = adeliverie->next;
      }
    }
    amessage = amessage->next;
  }
  return i;
}

void Results::indexhours ()
{ time_t aktzeit,lzeit;
  char zeit[40];
  char *temp;
  struct tm *fm;
  int anz;
  int hour;
  int day[2][24];
  int i;
  
  fm = new(tm);
  memcpy (fm,firstmessage(),sizeof(struct tm));
  fm->tm_sec=0;
  fm->tm_min=0;
  fm->tm_hour=0;
  aktzeit = mktime(fm);
  lzeit = mktime(lastmessage());
  hour = 0;
  while (aktzeit < lzeit) {
    temp = ctime(&aktzeit);
    strncpy (zeit,temp,strlen(temp)-1);
    zeit[strlen(temp)-1] = 0;
    anz = perhour(aktzeit);
    // den wert sichern
    day[0][hour] = anz;
    anz = d_perhour(aktzeit);
    day[1][hour] = anz;
    // Wenn die Stunde auf 24 steht dann ein Bild generieren
    hour++;
    if (hour == 24) {
      hour = 0;
      images->hourly_usage (day,fm);
      fm = localtime(&aktzeit);
      for (i=0;i<24;i++) { day[0][i] = 0; day[1][i] = 0; }
    }
    aktzeit = aktzeit + 3600;
  }
  // Hier auch noch den aktuellen Tag erstellen
  images->hourly_usage (day,fm);
}

// Zhlt die Anzahl der EMails fr das betreffende Monat
int Results::permonth (time_t startmonth,Domains *domain)
{ time_t start,end;
  MContainer *amessage=domain->messages;
  int i = 0;
  struct tm *temp=new(tm);
  
  start = startmonth;
  // Gehrt noch abgerundet
  temp = localtime(&start);
  temp->tm_sec = 0;
  temp->tm_min = 0;
  temp->tm_hour = 0;
  temp->tm_mday = 1;
  temp->tm_wday = 0;
  start = mktime(temp);
  // plus einem Monat
  if (temp->tm_mon == 11) {
    temp->tm_year++;
    temp->tm_mon=0;
  }
  else
    temp->tm_mon=temp->tm_mon  + 1;
  end = mktime(temp);
  // Jetzt die EMails zhlen die in diesem Zeitraum liegen
  while (amessage != NULL) {
    if ((mktime(amessage->message->startzeit) >= start) && (mktime(amessage->message->startzeit) < end))
      i++;
    amessage = amessage->next;
  }
  return i;

}

// Zhlt die Anzahl der Bytes fr das betreffende Monat
unsigned long Results::bytespermonth (time_t startmonth,Domains *domain)
{ time_t start,end;
  MContainer *amessage=domain->messages;
  unsigned long bytes = 0;
  struct tm *temp=new(tm);
  
  start = startmonth;
  // Gehrt noch abgerundet
  temp = localtime(&start);
  temp->tm_sec = 0;
  temp->tm_min = 0;
  temp->tm_hour = 0;
  temp->tm_mday = 1;
  temp->tm_wday = 0;
  start = mktime(temp);
  // plus einem Monat
  if (temp->tm_mon == 11) {
    temp->tm_year++;
    temp->tm_mon=0;
  }
  else
    temp->tm_mon=temp->tm_mon  + 1;
  end = mktime(temp);
  // Jetzt die EMails zhlen die in diesem Zeitraum liegen
  while (amessage != NULL) {
    if ((mktime(amessage->message->startzeit) >= start) && (mktime(amessage->message->startzeit) < end))
      bytes += amessage->message->bytes;
    amessage = amessage->next;
  }
  return bytes;

}

// Sammelt die Ergebnisse fr die Monatsdarstellung
char *Results::indexmonth (Domains *domain)
{ struct tm *start=new(tm);
  time_t *aktzeit=new(time_t);
  int month;
  
  time(aktzeit);
  start = localtime(aktzeit);
  domain->relyear = start->tm_year;
  for (month=0;month < 12;month++) {
    start->tm_sec = 0;
    start->tm_min = 0;
    start->tm_hour = 0;
    start->tm_mday = 1;
    start->tm_mon = month;
    *aktzeit = mktime(start);
    domain->monthusage[month] = domain->monthusage[month] + permonth(*aktzeit,domain);
    domain->monthbytes[month] = domain->monthbytes[month] + bytespermonth(*aktzeit,domain);
  }
  return images->month_usage(domain->monthusage,domain->domainname);
}

// Diese Funktion filtert aus der EMail Adresse den Domainnamen und den namen raus
void Results::splitemail (char *dname,char *name,char *email)
{ unsigned int i,offset;
  bool part1;
  
  // Die Funktion nimmt erst mal beinhart an das bereits gengend Speicher reserviert ist
//  printf ("%s\n",email);
  part1 = true;
  offset = 0;
  name[0] = 0;
  dname[0] = 0;    
  for (i=0;i<strlen(email);i++) {
    if ((part1) && (email[i] != '@'))
      name[i] = email[i];
    else if ((part1) && (email[i] == '@'))
    {
      // String abschliessen
      name[i] = 0;
      part1 = false;
      offset = i;
    }
    else if (!part1)
    {
      dname[i-offset-1] = email[i];
    }
  }
  // Den Domain Namen noch abschliessen
  dname[i-offset-1] = 0;
//  printf ("%s\n",dname);
}

char *Results::cleanname (char *name,LDomains *domain)
{
  // berprfen ob der unix username davor steht
  if (domain != NULL)
  {
    if (domain->unixuser != NULL) {
      if (strncmp(name,domain->unixuser,strlen(domain->unixuser)) == 0)
        strcpy (name,name+(strlen(domain->unixuser)+1));
    }
  }
  return NULL;
}

Domains *Results::getdomain (char *dname,LDomains *ldomain)
{ Domains *aktdomain=dWurzel;
  Domains *ndomain;
  char *vname;
  
  if (ldomain == NULL)
  {
    vname = (char *)malloc(6);
    strcpy (vname,"relay");
  }
  else
  {
    vname = (char *)malloc(strlen(dname)+1);
    strcpy (vname,dname);
  }
  while (aktdomain != NULL) {
    if (strncmp(aktdomain->domainname,vname,strlen(vname)) == 0)
      break;
    aktdomain = aktdomain->next;
  }
  if (aktdomain == NULL)
  {
    ndomain = adddomain();
    // berprfen ob local
    if (ldomain != NULL) {
      if (strcmp(dname,ldomain->domain)==0)
        ndomain->islocal=true;
      else
        ndomain->islocal=false;
    }
    // Die neue Domain in die Struktur einhngen
    aktdomain = dWurzel;
    if (aktdomain == NULL)
    {
      dWurzel = ndomain;
      aktdomain = ndomain;
    }
    else
    {
      while (aktdomain->next != NULL) {aktdomain = aktdomain->next;}
      aktdomain->next = ndomain;
      aktdomain = ndomain;
    }
    aktdomain->domainname = (char *)malloc(strlen(vname)+1);
    strcpy (aktdomain->domainname,vname);
  }
  free (vname);
  return aktdomain;
}

// Hier kommt nun das gesamt Domain Handling
void Results::ordne_zu ()
{ Message *aktmessage;
  Deliveries *aktdeliverie;
  char *dname = (char *)malloc(100);	// sollte gengen
  char *name = (char *)malloc(100);	// soltle auch gengen
  Domains *domain;
  Members *member;
  LDomains *ldomain;
  time_t aktzeit,tempzeit;
  
  // Hier werden die Nachrichten der Reihe nach durchgegangen
  // nur die bereits abgeschlossenen
  // und dann den domains -> members zugeordnet
  aktmessage = messages->mWurzel;
  while (aktmessage != NULL) {
    // Als erstes den Domain Namen fr die Nachricht ermitteln
    // Wenn tolocal dann ist es eine incoming und der recipient ist wichtig
    // Ansonsten ausgehend und der sender ist wichtig
    aktdeliverie = aktmessage->deliveries;
    if ((aktdeliverie != NULL) && (aktmessage->endmsg))
    {
      splitemail (dname,name,aktmessage->sender);
      // berprfen ob der sender von einer localen domain ist
      ldomain = ldWurzel;
      while (ldomain != NULL) {
        if (strcmp(ldomain->domain,dname) == 0)
          break;
        ldomain = ldomain->next;
      }
      if (ldomain == NULL)
        aktmessage->fromlocal = false;
      else
      {
        aktmessage->fromlocal = true;
      }
      splitemail (dname,name,aktdeliverie->recipient);
      // berprfen ob der empfnger von einer localen domain ist
      if (aktmessage->deliveries->tolocal)
      {
        ldomain = ldWurzel;
        while (ldomain != NULL) {
          if (strcmp(ldomain->domain,dname) == 0)
            break;
          ldomain = ldomain->next;
        }
      }
      if (aktmessage->deliveries->tolocal) {
        splitemail (dname,name,aktdeliverie->recipient);
      }
      else
      {
        splitemail (dname,name,aktmessage->sender);
      }
      // Jetzt die zustndige Domain Structur holen oder erzeugen
      domain = getdomain(dname,ldomain);
      // Jetzt den mail namen nachbearbeiten
      cleanname (name,ldomain);
      tempzeit = mktime(aktmessage->startzeit);
      memcpy (&aktzeit,&tempzeit,sizeof(time_t));
      // berprfen ob die aktzeit grer oder kleiner den Anfangs/Endzeiten ist
      if (domain->start != NULL)
      {
        if (mktime(domain->start) > aktzeit)
	  memcpy (domain->start,localtime(&aktzeit),sizeof(struct tm));
      }
      else
      {
        domain->start = new tm;
        memcpy (domain->start,localtime(&aktzeit),sizeof(struct tm));
      }    
      if (domain->end != NULL)
      {
        if (mktime(domain->end) < aktzeit)
	  memcpy (domain->end,localtime(&aktzeit),sizeof(struct tm));
      }
      else
      {
        domain->end = new tm;
        memcpy (domain->end,localtime(&aktzeit),sizeof(struct tm));
      }    
      // die Anzahl der EMails erhhen
      domain->anzahlm++;
      domain->bytes += aktmessage->bytes;
      if (aktmessage->deliveries->tolocal)
      {
        domain->incoming++;
	domain->incbytes += aktmessage->bytes;
	if (aktmessage->fromlocal)
	{
	  domain->local++;
	  domain->localbytes += aktmessage->bytes;
	}
      }
      else
      {
        domain->outgoing++;
	domain->outbytes += aktmessage->bytes;
      }
      // Den entsprechenden Sender/Empfnger hinzufgen
      // Also das selber Spielchen wie bei den Domains
      member = getmember(domain,name);
      // berprfen ob die aktzeit grer oder kleiner den Anfangs/Endzeiten ist
      if (member->start != NULL)
      {
        if (mktime(member->start) > aktzeit)
	  memcpy (member->start,localtime(&aktzeit),sizeof(struct tm));
      }
      else
      {
        member->start = new tm;
        memcpy (member->start,localtime(&aktzeit),sizeof(struct tm));
      }    
      if (member->end != NULL)
      {
        if (mktime(member->end) < aktzeit)
	  memcpy (member->end,localtime(&aktzeit),sizeof(struct tm));
      }
      else
      {
        member->end = new tm;
        memcpy (member->end,localtime(&aktzeit),sizeof(struct tm));
      }    

      // Jetzt die Kontrollwerte fr diesen Benutzer erhhen
      member->anzahlm++;
      member->bytes += aktmessage->bytes;
      if (aktmessage->deliveries->tolocal)
      {
        member->incoming++;
	member->incbytes +=  aktmessage->bytes;
	if (aktmessage->fromlocal)
	{
	  member->local++;
	  member->localbytes += aktmessage->bytes;
	}
      }
      else
      {
        member->outgoing++;
	member->outbytes += aktmessage->bytes;
      }
      // Die NAchricht zu den Domains dazuhngen
      AddMessageToDomain (domain,aktmessage);
    }
    aktmessage = aktmessage->next;
  }
  // Hier ist die Zuordnung zu den Domains abgeschlossen
  // Jetzt kommen noch ein paar allgemeine Berechnungen
  // die diff sec Berechnung
  domain = dWurzel;
  while (domain != NULL) {
    if ((domain->end != NULL) && (domain->start!=NULL)) {
      tempzeit = mktime(domain->end);
      memcpy (&aktzeit,&tempzeit,sizeof(time_t));
      domain->sec = aktzeit - mktime(domain->start);
    }
    else
      domain->sec = 0;
    member = domain->members;
    while (member != NULL) {
      if ((member->start != NULL) && (member->end != NULL)) {
        tempzeit = mktime(member->end);
        memcpy (&aktzeit,&tempzeit,sizeof(time_t));
        member->sec = aktzeit - mktime(member->start);      
      }
      else
        member->sec = 0;
      member = member->next;
    }
    domain = domain->next;
  }
  free (dname);
  free (name);
}

// Funktion berprft ob in der Domne bereits ein user mit name vorhanden ist
// Wenn nicht wird er erzeugt und die Structur zurckgeliefert
Members *Results::getmember (Domains *domain,char *name)
{ Members *aktmember;
  Members *nmember;
  
  aktmember = domain->members;
  while (aktmember != NULL) {
    if (strcmp(aktmember->name,name) == 0)
      break;
    aktmember = aktmember->next;
  }
  if (aktmember == NULL)
  {
    // Ist noch nicht dabei
    nmember = addmember();
    nmember->name = (char *)malloc(strlen(name)+1);
    strcpy (nmember->name,name);
    // erzeugt und einhngen
    aktmember = domain->members;
    if (aktmember == NULL)
    {
      aktmember = nmember;
      domain->members = nmember;
    }
    else
    {
      while (aktmember->next != NULL) {aktmember = aktmember->next;}
      aktmember->next = nmember;
      aktmember = nmember;
    }
  }
  return aktmember;
}

void Results::AddMessageToDomain (Domains *domain,Message *message)
{ MContainer *nelement = new (MContainer);

  nelement->next = NULL;
  nelement->message = message;
  if (domain->messages == NULL)
  {
    domain->messages = nelement;
    domain->lmessage = nelement;
  }
  else
  {
    if (domain->lmessage != NULL)
    {
      domain->lmessage->next = nelement;
      domain->lmessage = domain->lmessage->next;
    }
  }
}

Domains *Results::adddomain()
{ Domains *ndomain = new Domains;
  int i;
  
  ndomain->domainname = NULL;
  ndomain->messages = NULL;
  ndomain->anzahlm = 0;
  ndomain->incoming = 0;
  ndomain->outgoing = 0;
  ndomain->incbytes = 0;
  ndomain->outbytes = 0;
  ndomain->anzahld = 0;
  ndomain->bytes = 0;
  ndomain->start = NULL;
  ndomain->end = NULL;
  ndomain->sec = 0;
  ndomain->members = NULL;
  ndomain->next = NULL;
  ndomain->local = 0;
  ndomain->localbytes = 0;
  for (i=0;i<12;i++) {
    ndomain->monthusage[i] = 0;
    ndomain->monthbytes[i] = 0;
  }
  return ndomain;;
}

Members *Results::addmember()
{ Members *nmember = new Members;

  nmember->name = NULL;
  nmember->messages = NULL;
  nmember->anzahlm = 0;
  nmember->incoming = 0;
  nmember->outgoing = 0;
  nmember->incbytes = 0;
  nmember->outbytes = 0;
  nmember->anzahld = 0;
  nmember->bytes = 0;
  nmember->start = NULL;
  nmember->end = NULL;
  nmember->sec = 0;
  nmember->next = NULL;
  nmember->local = 0;
  nmember->localbytes = 0;
  return nmember;
}

void Results::load_virtual_domains ()
{ FILE *f;
  char zeile[100];
  char *dname;
  char *name;
  LDomains *aktDomain=NULL;
  LDomains *nDomain;
  int pos;
  
  // Die Datei mte eigentlich immer in /var/qmail/control/ liegen
  if  (config->vdomains != NULL) {
    f = fopen (config->vdomains,"r");
    while (!feof(f)) {
      zeile [0] = 0;
      fgets (zeile,100,f);
      if (strlen(zeile) > 3)
      {
        // Den Inhalt analysieren
        // d.h den domainnamen und den zustndigen unix user rauskratzen
	if ((pos = strpos(zeile, ':')) < 0)
		exit (1);	// Invalid format
	if ((dname = (char *)malloc(pos+1)) == NULL)
		exit (1);	// Invalid format
	name = (char *)malloc(strlen(zeile) - pos - 1);
	strncpy (name, &zeile[pos+1], strlen(zeile) - pos - 2);
        strncpy (dname,zeile,strpos(zeile,':'));
        dname[pos] = 0;
        name[strlen(zeile)-pos-2] = 0;
        // Und die ermittelten Werte sichern
        nDomain = new LDomains;
        nDomain->next = NULL;
        nDomain->unixuser = name;
        nDomain->domain = dname;
        if (aktDomain == NULL) {
          aktDomain = nDomain;
       	  ldWurzel = nDomain;
        }
        else
        {
          aktDomain->next = nDomain;
	  aktDomain = nDomain;
        }
      }
    }
    // ressourcen wieder freigeben
    fclose (f);
    // Die Domainnamen ausgeben
    aktDomain = ldWurzel;
    while (aktDomain != NULL) {
      printf ("Domain: %s UnixUser: %s\n",aktDomain->domain,aktDomain->unixuser);
      aktDomain = aktDomain->next;
    }    
  }
  else
  {
    printf ("Ubernehme die domains aus qmailalizer.conf\n");
    ldWurzel = config->ldWurzel;
  }
}

void Results::save_results (XMLOutput *xmloutput)
{ LDomains *ldomain;
  Domains *domain;
  int number=0;
  char *temp=(char *)malloc(3);
  
  // Sichert die Ergebnisse in die xml Datei
  // Speichert nur die localen Domains
  // Die Bilder
  images->domain_view (dWurzel);
  // Fr jede Doamin
  domain = dWurzel;
  while (domain != NULL)
  {
    ldomain = ldWurzel;
    while (ldomain != NULL) {
      if (strcmp(ldomain->domain,domain->domainname) == 0)
        break;
      ldomain = ldomain->next;
    }
    if ((ldomain != NULL) || (strcmp(domain->domainname,"relay") == 0))
    {
      // Es ist also eine locale
      // Also die Werte sichern
      number++;
      domain->link = (char *)malloc(strlen(config->phpname)+strlen("./?number=")+3);
      strcpy (domain->link,"./");
      strcat (domain->link,config->phpname);
      strcat (domain->link,"?number=");
      temp = itoa(number,temp,10);
      strcat (domain->link,temp);
    domain->filename = indexmonth(domain);
      
      xmloutput->AddDomain(domain);
    }
    domain = domain->next;
  } 
  free (temp);
}

Domains *Results::AddDomain (char *domainname,unsigned long anzahlm,unsigned long incoming,unsigned long outgoing,unsigned long incbytes,unsigned long outbytes,unsigned long local,unsigned long localbytes,unsigned long anzahld,unsigned long bytes,unsigned long sec,struct tm *start,struct tm *end,unsigned long usage[12],unsigned long monthbytes[12])
{ Domains *aktdomain;
  LDomains *ldomain;
  int i;
  
  ldomain = ldWurzel;
  while (ldomain != NULL) {
    if (strcmp(ldomain->domain,domainname) == 0)
      break;
    ldomain = ldomain->next;
  }
  aktdomain = getdomain (domainname,ldomain);
  aktdomain->anzahlm = anzahlm;
  aktdomain->incoming = incoming;
  aktdomain->outgoing = outgoing;
  aktdomain->incbytes = incbytes;
  aktdomain->outbytes = outbytes;
  aktdomain->local = local;
  aktdomain->localbytes = localbytes;
  aktdomain->anzahld = anzahld;
  aktdomain->bytes = bytes;
  aktdomain->sec = sec;
  aktdomain->start = new (struct tm);
  aktdomain->next = NULL;
  aktdomain->messages = NULL;
  aktdomain->lmessage = NULL;
  memcpy (aktdomain->start,start,sizeof(struct tm));
  aktdomain->end = new (struct tm);
  memcpy (aktdomain->end,end,sizeof(struct tm));
  for (i=0;i<12;i++) {
    aktdomain->monthusage[i] = usage[i];
    aktdomain->monthbytes[i] = monthbytes[i];
  }
  return aktdomain;
}

void Results::AddMember (Domains *domain,char *name,unsigned long anzahlm,unsigned long incoming,unsigned long outgoing,unsigned long incbytes,unsigned long outbytes,unsigned long local,unsigned long localbytes,unsigned long anzahld,unsigned long bytes,unsigned long sec,struct tm *start,struct tm *end)
{ Members *aktmember;
  
  aktmember = getmember (domain,name);
  aktmember->anzahlm = anzahlm;
  aktmember->incoming = incoming;
  aktmember->outgoing = outgoing;
  aktmember->incbytes = incbytes;
  aktmember->outbytes = outbytes;
  aktmember->local = local;
  aktmember->localbytes = localbytes;
  aktmember->anzahld = anzahld;
  aktmember->bytes = bytes;
  aktmember->sec = sec;
  aktmember->start = new (struct tm);
  aktmember->next = NULL;
  aktmember->messages = NULL;
  
  memcpy (aktmember->start,start,sizeof(struct tm));
  aktmember->end = new (struct tm);
  memcpy (aktmember->end,end,sizeof(struct tm));
}

void Results::ChangeUsage (Domains *domain,unsigned long usage[12],unsigned long bytes[12])
{ int i;

  for (i=0;i<12;i++) {
    domain->monthusage[i] = usage[i];
    domain->monthbytes[i] = bytes[i];
  }
}

#endif
